from setuptools import setup

setup(
    name='gmlutil',
    version='0.9.18',    
    description='General Machine Learning Utility Package',
    url='https://github.com/Phillip1982/gmlutil',
    author='Phillip Kim',
    author_email='phillip.kim@ejgallo.com',
    license='BSD 2-clause', ## Change this
    packages=['gmlutil'],
    install_requires=[ # package>=0.2,<0.3
    	'aiobotocore==2.5.0',
    	'awscli==1.27.76',
		'boto3==1.26.76', # 1.20.24', 
		'botocore==1.29.76', # 1.23.24',
		'dash>=2.0.0',
		'datetime>=4.3',
        'evidently>=0.4.8',
        'fastapi>=0.100.0',
        'fastapi-restful>=0.5.0',
		'folium>=0.14.0',
		'fuzzywuzzy>=0.18.0',
		'geopandas>=0.10.2',
		'hana-ml',
		'imbalanced-learn>=0.8.0',
        'iterative-telemetry>=0.0.5',
		'jupyter_dash>=0.4.0',
		'kmodes>=0.11.0',
        'nltk>=3.6.7',
		'numpy>=1.19.5,<1.23.0',
        'oracledb>=1.4.2',
		'pandas>=1.3.5,<1.4.3',
        'packaging>=21.3',
		'plotly>=5.5.0',
		'psycopg2-binary>=2.9.1',
        'pyarrow<11.0dev',
        'pydantic<2',
		'pyhdb>=0.3.4',
		'pytest-astropy',
        'python-levenshtein-wheels>=0.13.1',
		'pytrends>=4.7.3',
        'PyYAML>=5.1',
        'redshift_connector>=2.0.916',
        'requests>=2.21.0',
        'rich>=13',
		'rsa<4.8,>=3.1.2',
		'sagemaker==2.0.0',
		'sagemaker-data-insights==0.3.0',
        'scikit-learn>=0.24.0',
        'scipy>=1.5.4',
        'seaborn>=0.11.2',
		'sparkmagic>=0.20.4',
		'sqlalchemy-redshift>=0.8.6',
		'statsmodels>=0.12.2',
		's3transfer>=0.6.0', # 0.5.0',
        'typer>=0.3',
		'typing_extensions>=4.0',
        'typing-inspect>=0.9.0',
        'uvicorn>=0.22.0',
        'watchdog>=3',
		'xgboost>=1.3.3'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: IPython',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
		'Operating System :: MacOS',
		'Operating System :: Microsoft :: Windows :: Windows XP',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ],
)
